/*
 * cistpl.h v$Revision: 1.9 $: $Date: 1994/08/24 20:31:05 $ (David Hinds)
 */

#ifndef _LINUX_CISTPL_H
#define _LINUX_CISTPL_H

#define CISTPL_NULL		0x00
#define CISTPL_DEVICE		0x01
#define CISTPL_VERS_1		0x15
#define CISTPL_CONFIG		0x1a
#define CISTPL_CFTABLE_ENTRY	0x1b
#define CISTPL_MANUFACTURER	0x20
#define CISTPL_END		0xff

#define CISTPL_VERS_1_MAX_PROD_STRINGS	4
#define CISTPL_VERS_1_STRLEN		60

typedef struct cistpl_vers_1_t {
    u_char	major;
    u_char	minor;
    int		ns;
    char	pi[CISTPL_VERS_1_MAX_PROD_STRINGS][CISTPL_VERS_1_STRLEN];
} cistpl_vers_1_t;

typedef struct cistpl_config {
    u_char	last_idx;
    u_long	base;
    u_long	rmask[4];
} cistpl_config_t;

typedef struct cistpl_power_t {
    u_char	present;
    u_char	p[8];
} cistpl_power_t;

#define CISTPL_TIMING_WAIT		0x03
#define CISTPL_TIMING_RDY		0x1c
#define CISTPL_TIMING_RDY_SHIFT		2
#define CISTPL_TIMING_RSV		0xe0
#define CISTPL_TIMING_RSV_SHIFT		5

typedef struct cistpl_timing_t {
    u_char	scale;
    u_char	wait_speed;
    u_char	rdy_speed;
    u_char	rsv_speed;
} cistpl_timing_t;

#define CISTPL_IO_LINES_MASK	0x1f
#define CISTPL_IO_8BIT		0x20
#define CISTPL_IO_16BIT		0x40
#define CISTPL_IO_RANGE		0x80

typedef struct cistpl_io_t {
    u_char	flags;
    u_char	num;
    u_long	base[16];
    u_long	len[16];
} cistpl_io_t;

typedef struct cistpl_irq_t {
    u_long IRQInfo1, IRQInfo2;
} cistpl_irq_t;

#define CISTPL_CFTABLE_ENTRY_POWER	0x03
#define CISTPL_CFTABLE_ENTRY_TIMING	0x04
#define CISTPL_CFTABLE_ENTRY_IO      	0x08
#define CISTPL_CFTABLE_ENTRY_IRQ	0x10
#define CISTPL_CFTABLE_ENTRY_MEM	0x60
#define CISTPL_CFTABLE_ENTRY_MISC	0x80

typedef struct cistpl_cftable_entry_t {
    u_char		index, Default;
    u_char		interface;
    u_char		features;
    cistpl_power_t	power[4];
    cistpl_timing_t	timing;
    cistpl_io_t		io;
    cistpl_irq_t	irq;
} cistpl_cftable_entry_t;

typedef union cisparse_t {
    cistpl_vers_1_t	version_1;
    cistpl_config_t	config;
    cistpl_cftable_entry_t cftable_entry;
} cisparse_t;

#ifdef __KERNEL__
int parse_tuple(client_handle_t handle, tuple_t *tuple,
		cisparse_t *parse);
#endif /* __KERNEL__ */

#endif /* LINUX_CISTPL_H */
