/*
 * In the absence of a suitable I/O and memory resource manager,
 * this will have to do.
 */

#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/types.h>

#include "iohack.h"

/* Each address registrar is an ordered linked list */
static resource_entry_t registrar[HW_NTYPES] = {
    { 0, 0, NULL, NULL }, /* ... */
};

static resource_entry_t *find_gap(resource_entry_t *root,
				  resource_entry_t *entry)
{
    unsigned long flags;
    resource_entry_t *p;
    
    if (entry->base > entry->base+entry->num-1)
	return NULL;
    save_flags(flags);
    cli();
    for (p = root; ; p = p->next) {
	if ((p != root) && (p->base+p->num-1 >= entry->base)) {
	    p = NULL;
	    break;
	}
	if ((p->next == NULL) ||
	    (p->next->base > entry->base+entry->num-1))
	    break;
    }
    restore_flags(flags);
    return p;
}

int check_resource(resource_t type, resource_entry_t *entry)
{
    return (find_gap(&registrar[type], entry) == NULL) ? -EBUSY : 0;
}

int alloc_resource(resource_t type, resource_entry_t *entry)
{
    unsigned long flags;
    resource_entry_t *p;
    
    save_flags(flags);
    cli();
    p = find_gap(&registrar[type], entry);
    if (p == NULL) {
	restore_flags(flags);
	return -EBUSY;
    }
    entry->next = p->next;
    p->next = entry;
    restore_flags(flags);
    return 0;
}

int free_resource(resource_t type, resource_entry_t *entry)
{
    unsigned long flags;
    resource_entry_t *p;
    
    save_flags(flags);
    cli();
    for (p = &registrar[type]; p->next != entry; p = p->next) {
	if (p->next == NULL) {
	    restore_flags(flags);
	    return -EINVAL;
	}
    }
    p->next = entry->next;
    restore_flags(flags);
    return 0;
}

#if 0

int get_resource_list(resource_t type, char *buf)
{
    int len = 0;
    resource_entry_t *p;

    for (p = &registrar[type]; p; p = p->next) {
	len += sprintf(buf+len, "[%#6lx-%#6lx]: %s\n",
		       p->base, p->base+len, p->dev);
    }
    return len;
}

#endif
